@@ -22,7 +22,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: s_misc.c,v 1.24 1998/08/05 02:39:04 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: s_misc.c,v 1.26 1999/01/13 02:14:36 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -164,6 +164,7 @@ aClient	*sptr;
 **	to internal buffer (nbuf). *NEVER* use the returned pointer
 **	to modify what it points!!!
 */
+
 char	*get_client_name(sptr, showip)
 aClient *sptr;
 int	showip;
@@ -188,7 +189,13 @@ int	showip;
 					sptr->name, USERLEN,
 					(!(sptr->flags & FLAGS_GOTID)) ? "" :
 					sptr->auth,
+#ifdef INET6 
+					      inetntop(AF_INET6,
+						       (char *)&sptr->ip,
+						       mydummy, MYDUMMY_SIZE));
+#else
 					      inetntoa((char *)&sptr->ip));
+#endif
 			else
 			    {
 				if (mycmp(sptr->name, sptr->sockhost))
@@ -243,6 +250,7 @@ Reg	char	*host;
 	else
 		s = host;
 	strncpyzt(cptr->sockhost, s, sizeof(cptr->sockhost));
+	Debug((DEBUG_DNS,"get_sockhost %s",s));
 }
 
 /*
@@ -989,6 +997,9 @@ char	*name;
 		   sp->is_ckr, sp->is_cbr, sp->is_skr, sp->is_sbr);
 	sendto_one(cptr, ":%s %d %s :time connected %u %u",
 		   ME, RPL_STATSDEBUG, name, sp->is_cti, sp->is_sti);
+#if defined(USE_IAUTH)
+	report_iauth_stats(cptr, name);
+#endif
 }
 
 #ifdef CACHED_MOTD
