@@ -22,7 +22,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: s_serv.c,v 1.51 1998/11/12 11:44:52 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: s_serv.c,v 1.53 1999/01/28 23:50:17 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -838,6 +838,8 @@ Reg	aClient	*cptr;
 #endif
 	sendto_flag(SCH_SERVER, "Sending SERVER %s (%d %s)", cptr->name,
 		    1, cptr->info);
+	sendto_flag(SCH_DEBUG, "Burst to %s: %X%s", inpath,
+		    cptr->hopcount, (cptr->flags & FLAGS_ZIP) ? "z" : "");
 	/*
 	** Old sendto_serv_but_one() call removed because we now
 	** need to send different names to different servers
@@ -860,7 +862,6 @@ Reg	aClient	*cptr;
 			sendto_one(acptr,":%s SERVER %s 2 %s :%s",
 				   ME, cptr->name, stok, cptr->info);
 	    }
-
 	/*
 	** Pass on my client information to the new server
 	**
@@ -901,6 +902,9 @@ Reg	aClient	*cptr;
 				   acptr->hopcount+1, stok, acptr->info);
 	    }
 
+	sendto_flag(SCH_DEBUG, "SERVER phase complete: %u",
+		    (int)DBufLength(&cptr->sendQ));
+
 	for (acptr = &me; acptr; acptr = acptr->prev)
 	    {
 		/* acptr->from == acptr for acptr == cptr */
@@ -948,6 +952,10 @@ Reg	aClient	*cptr;
 		    }
 		/* the previous if does NOT catch all services.. ! */
 	    }
+
+	sendto_flag(SCH_DEBUG, "client phase complete: %u",
+		    (int)DBufLength(&cptr->sendQ));
+
 	/*
 	** Last, pass all channels modes
 	** only sending modes for LIVE channels.
@@ -962,6 +970,10 @@ Reg	aClient	*cptr;
 				send_channel_modes(cptr, chptr);
 			    }
 	    }
+
+	sendto_flag(SCH_DEBUG, "NJOIN/mode phase complete: %u",
+		    (int)DBufLength(&cptr->sendQ));
+
 	cptr->flags &= ~FLAGS_CBURST;
 #ifdef	ZIP_LINKS
  	/*
@@ -1320,8 +1332,8 @@ char	*to;
 			else
 				(void)strcpy(buf, tmp->name);
 			sendto_one(sptr, rpl_str(RPL_STATSPING, to),
-				   buf, cp->lseq, cp->lrecv,
-				   cp->ping / (cp->recv ? cp->recv : 1),
+				   buf, cp->lseq, cp->lrecvd,
+				   cp->ping / (cp->recvd ? cp->recvd : 1),
 				   tmp->pref);
 			sendto_flag(SCH_DEBUG, "%s: %d", buf, cp->seq);
 		    }
